@@ -23,10 +23,10 @@ from jax.interpreters import xla
 from jax._src.util import prod
 from jax._src import dtypes
 from jax import lax
-from jax.lib import xla_client
 from jax.interpreters import ad
 from jax.interpreters import batching
-from jax.lib import pocketfft
+from jax._src.lib import xla_client
+from jax._src.lib import pocketfft
 
 xops = xla_client.ops
 
@@ -61,7 +61,7 @@ def fft_impl(x, fft_type, fft_lengths):
   return xla.apply_primitive(fft_p, x, fft_type=fft_type, fft_lengths=fft_lengths)
 
 _complex_dtype = lambda dtype: (np.zeros((), dtype) + np.zeros((), np.complex64)).dtype
-_real_dtype = lambda dtype: np.zeros((), dtype).real.dtype
+_real_dtype = lambda dtype: np.finfo(dtype).dtype
 _is_even = lambda x: x % 2 == 0
 
 def fft_abstract_eval(x, fft_type, fft_lengths):
