@@ -14,6 +14,7 @@
 # limitations under the License.
 
 import functools
+from functools import partial
 
 import numpy as np
 
@@ -22,26 +23,26 @@ from jax._src.numpy.vectorize import vectorize
 from jax._src import ad_util
 from jax._src import api
 from jax import lax
-from jax import ops
 from jax._src import dtypes
 from jax.interpreters import xla
 from jax.interpreters import ad
 from jax.interpreters import batching
-from jax._src.util import partial, prod
+from jax._src.util import prod
 from jax.core import Primitive, ShapedArray, raise_to_shaped
 from jax._src.lax.lax import (
     standard_primitive, standard_unop, naryop_dtype_rule, _float, _complex,
     _input_dtype, _broadcasting_select)
 from jax._src.lax import lax as lax_internal
-from jax.lib import lapack
+from jax._src.lib import lapack
 
-from jax.lib import cuda_linalg
-from jax.lib import cusolver
-from jax.lib import cusparse
-from jax.lib import rocsolver
+from jax._src.lib import cuda_linalg
+from jax._src.lib import cusolver
+from jax._src.lib import cusparse
+from jax._src.lib import rocsolver
 
-from jax.lib import xla_client
-from jax.lib import xla_bridge as xb
+from jax._src.lib import xla_client
+from jax._src.lib import xla_bridge as xb
+from jax._src.lib import version as jaxlib_version
 
 xops = xla_client.ops
 
@@ -88,7 +89,7 @@ def eig(x, compute_left_eigenvectors=True, compute_right_eigenvectors=True):
 def eigh(x, lower: bool = True, symmetrize_input: bool = True):
   """Eigendecomposition of a Hermitian matrix.
 
-  Computes the eigenvalues and eigenvectors of a complex Hermitian or real
+  Computes the eigenvectors and eigenvalues of a complex Hermitian or real
   symmetric square matrix.
 
   Args:
@@ -662,7 +663,8 @@ def triangular_solve_batching_rule(batched_args, batch_dims, left_side,
                             unit_diagonal=unit_diagonal), 0
 
 def _triangular_solve_translation_rule(
-    c, a, b, *, left_side, lower, transpose_a, conjugate_a, unit_diagonal):
+    ctx, avals_in, avals_out, a, b, *, left_side, lower, transpose_a,
+    conjugate_a, unit_diagonal):
   if conjugate_a and not transpose_a:
     a = xops.Conj(a)
     conjugate_a = False
@@ -671,7 +673,8 @@ def _triangular_solve_translation_rule(
   else:
     transpose = (xops.TriangularSolveOptions_Transpose.ADJOINT if conjugate_a
                  else xops.TriangularSolveOptions_Transpose.TRANSPOSE)
-  return xops.TriangularSolve(a, b, left_side, lower, unit_diagonal, transpose)
+  return [
+      xops.TriangularSolve(a, b, left_side, lower, unit_diagonal, transpose)]
 
 triangular_solve_p = standard_primitive(
     triangular_solve_shape_rule, triangular_solve_dtype_rule,
@@ -751,8 +754,8 @@ def _lu_pivots_body_fn(i, permutation_and_swaps):
   iotas = jnp.ix_(*(lax.iota(jnp.int32, b) for b in batch_dims))
   x = permutation[..., i]
   y = permutation[iotas + (j,)]
-  permutation = ops.index_update(permutation, ops.index[..., i], y)
-  return ops.index_update(permutation, ops.index[iotas + (j,)], x), swaps
+  permutation = permutation.at[..., i].set(y)
+  return permutation.at[iotas + (j,)].set(x), swaps
 
 
 @partial(api.jit, static_argnums=(1,))
@@ -853,16 +856,13 @@ def _lu_unblocked(a):
     else:
       magnitude = jnp.abs(a[:, k])
     i = jnp.argmax(jnp.where(m_idx >= k, magnitude, -jnp.inf))
-    pivot = ops.index_update(pivot, ops.index[k], i)
-
-    a = ops.index_update(a, ops.index[[k, i],], a[[i, k],])
-
-    perm = ops.index_update(perm, ops.index[[i, k],], perm[[k, i],])
+    pivot = pivot.at[k].set(i)
+    a = a.at[[k, i],].set(a[[i, k],])
+    perm = perm.at[[i, k],].set(perm[[k, i],])
 
     # a[k+1:, k] /= a[k, k], adapted for loop-invariant shapes
     x = a[k, k]
-    a = ops.index_update(a, ops.index[:, k],
-                         jnp.where(m_idx > k, a[:, k] / x, a[:, k]))
+    a = a.at[:, k].set(jnp.where(m_idx > k, a[:, k] / x, a[:, k]))
 
     # a[k+1:, k+1:] -= jnp.outer(a[k+1:, k], a[k, k+1:])
     a = a - jnp.where((m_idx[:, None] > k) & (n_idx > k),
@@ -888,20 +888,17 @@ def _lu_blocked(a, block_size=128):
     b = min(r - k, block_size)
     block_pivot, block_perm, lu_block = _lu_unblocked(a[k:, k:k+b])
 
-    pivot = ops.index_update(pivot, ops.index[k:k+b], block_pivot + k)
-    perm = ops.index_update(perm, ops.index[k:], perm[block_perm + k])
-    a = ops.index_update(a, ops.index[k:, :], a[block_perm + k, :])
-    a = ops.index_update(a, ops.index[k:, k:k+b], lu_block)
+    pivot = pivot.at[k:k+b].set(block_pivot + k)
+    perm = perm.at[k:].set(perm[block_perm + k])
+    a = a.at[k:, :].set(a[block_perm + k, :])
+    a = a.at[k:, k:k+b].set(lu_block)
 
     if k + b < n:
-      a = ops.index_update(
-        a, ops.index[k:k+b, k+b:],
-        triangular_solve(a[k:k+b, k:k+b], a[k:k+b, k+b:],
-                         left_side=True, lower=True, unit_diagonal=True))
-      a = ops.index_add(
-        a, ops.index[k+b:, k+b:],
-        -lax.dot(a[k+b:, k:k+b], a[k:k+b, k+b:],
-                 precision=lax.Precision.HIGHEST))
+      a = a.at[k:k+b, k+b:].set(
+        triangular_solve(a[k:k+b, k:k+b], a[k:k+b, k+b:], left_side=True,
+                         lower=True, unit_diagonal=True))
+      a = a.at[k+b:, k+b:].add(-lax.dot(a[k+b:, k:k+b], a[k:k+b, k+b:],
+                                        precision=lax.Precision.HIGHEST))
   return a, pivot, perm
 
 def _lu_python(x):
@@ -1260,8 +1257,8 @@ def svd_jvp_rule(primals, tangents, full_matrices, compute_uv):
   if not compute_uv:
     return (s,), (ds,)
 
-  s_diffs = jnp.square(s_dim) - jnp.square(_T(s_dim))
-  s_diffs_zeros = jnp.eye(s.shape[-1], dtype=A.dtype)  # jnp.ones((), dtype=A.dtype) * (s_diffs == 0.)  # is 1. where s_diffs is 0. and is 0. everywhere else
+  s_diffs = (s_dim + _T(s_dim)) * (s_dim - _T(s_dim))
+  s_diffs_zeros = jnp.eye(s.shape[-1], dtype=s.dtype)  # jnp.ones((), dtype=A.dtype) * (s_diffs == 0.)  # is 1. where s_diffs is 0. and is 0. everywhere else
   F = 1 / (s_diffs + s_diffs_zeros) - s_diffs_zeros
   dSS = s_dim * dS  # dS.dot(jnp.diag(s))
   SdS = _T(s_dim) * dS  # jnp.diag(s).dot(dS)
@@ -1447,3 +1444,119 @@ def tridiagonal_solve(dl, d, du, b):
     raise ValueError(f'Only f32/f64 are supported, got {t}')
 
   return tridiagonal_solve_p.bind(dl, d, du, b, m=m, n=n, ldb=ldb, t=t)
+
+
+# Schur Decomposition
+
+
+def schur(x,
+          compute_schur_vectors=True,
+          sort_eig_vals=False,
+          select_callable=None):
+  return schur_p.bind(
+      x,
+      compute_schur_vectors=compute_schur_vectors,
+      sort_eig_vals=sort_eig_vals,
+      select_callable=select_callable)
+
+
+def _schur_impl(operand, *, compute_schur_vectors, sort_eig_vals,
+                select_callable):
+  return xla.apply_primitive(
+      schur_p,
+      operand,
+      compute_schur_vectors=compute_schur_vectors,
+      sort_eig_vals=sort_eig_vals,
+      select_callable=select_callable)
+
+
+def _schur_translation_rule(c, operand, *, compute_schur_vectors,
+                            sort_eig_vals):
+  raise NotImplementedError(
+      "Schur decomposition is only implemented on the CPU backend.")
+
+
+def _schur_abstract_eval(operand, *, compute_schur_vectors, sort_eig_vals,
+                         select_callable):
+
+  if operand.ndim < 2 or operand.shape[-2] != operand.shape[-1]:
+    raise ValueError("Argument to Schur decomposition must have "
+                     "shape [..., n, n], got shape {}".format(operand.shape))
+
+  batch_dims = operand.shape[:-2]
+  n = operand.shape[-1]
+  dtype = operand.dtype
+  dtype = dtypes.canonicalize_dtype(dtype)
+  T = operand.update(shape=batch_dims + (n, n), dtype=dtype)
+  vs = operand.update(shape=batch_dims + (n, n), dtype=dtype)
+
+  return (T, vs) if compute_schur_vectors else (T,)
+
+
+def _schur_cpu_translation_rule(c, operand, *, compute_schur_vectors,
+                                sort_eig_vals, select_callable):
+  shape = c.get_shape(operand)
+  batch_dims = shape.dimensions()[:-2]
+
+  if jaxlib_version < (0, 1, 72):
+    raise NotImplementedError(
+        "The Schur primitive is only implemented for jaxlib versions >= 0.1.72"
+    )
+
+  _cpu_gees = lapack.gees
+
+  if sort_eig_vals:
+    T, vs, sdim, info = _cpu_gees(
+        c,
+        operand,
+        jobvs=compute_schur_vectors,
+        sort=sort_eig_vals,
+        select=select_callable)
+  else:
+    T, vs, info = _cpu_gees(
+        c,
+        operand,
+        jobvs=compute_schur_vectors,
+        sort=sort_eig_vals,
+        select=select_callable)
+
+  ok = xops.Eq(info, xops.ConstantLiteral(c, np.array(0, np.int32)))
+  T = _broadcasting_select(c, xops.Reshape(ok, batch_dims + (1, 1)), T,
+                           _nan_like(c, T))
+  output = [T]
+  if compute_schur_vectors:
+    vs = _broadcasting_select(c, xops.Reshape(ok, batch_dims + (1, 1)), vs,
+                              _nan_like(c, vs))
+
+    output.append(vs)
+
+  return xops.Tuple(c, output)
+
+
+def _schur_batching_rule(batched_args, batch_dims, *, compute_schur_vectors,
+                         sort_eig_vals, select_callable):
+  x, = batched_args
+  bd, = batch_dims
+  x = batching.moveaxis(x, bd, 0)
+
+  return schur_p.bind(
+      x,
+      compute_schur_vectors=compute_schur_vectors,
+      sort_eig_vals=sort_eig_vals,
+      select_callable=select_callable), (0,) * (1 + compute_schur_vectors)
+
+
+def _schur_jvp_rule(primals, tangents, *, compute_schur_vectors, sort_eig_vals):
+  raise NotImplementedError(
+      'The differentiation rules for the Schur factorization have not been implemented.'
+  )
+
+
+schur_p = Primitive('schur')
+schur_p.multiple_results = True
+schur_p.def_impl(_schur_impl)
+schur_p.def_abstract_eval(_schur_abstract_eval)
+xla.translations[schur_p] = _schur_translation_rule
+xla.backend_specific_translations['cpu'][schur_p] = _schur_cpu_translation_rule
+batching.primitive_batchers[schur_p] = _schur_batching_rule
+ad.primitive_jvps[schur_p] = _schur_jvp_rule
