@@ -28,8 +28,9 @@ from jax.config import config
 from jax.core import NamedShape
 from jax._src.api import jit, vmap
 from jax._src.numpy.lax_numpy import (_arraylike, _check_arraylike,
-                                      _constant_like, _convert_and_clip_integer)
-from jax.lib import xla_bridge
+                                      _constant_like, _convert_and_clip_integer,
+                                      _canonicalize_axis)
+from jax._src.lib import xla_bridge
 from jax.numpy.linalg import cholesky, svd, eigh
 from jax.interpreters import ad
 from jax.interpreters import batching
@@ -71,7 +72,7 @@ def _check_prng_key(key):
           'Raw arrays as random keys to jax.random functions are deprecated. '
           'Assuming valid threefry2x32 key for now.',
           FutureWarning)
-    return prng.PRNGKeyArray(prng.threefry_prng_impl, key), True
+    return prng.PRNGKeyArray(_get_default_prng_impl(), key), True
   else:
     raise TypeError(f'unexpected PRNG key type {type(key)}')
 
@@ -81,28 +82,75 @@ def _return_prng_keys(was_wrapped, key):
   if config.jax_enable_custom_prng:
     return key
   else:
-    return key.keys if was_wrapped else key
+    return key.unsafe_raw_array() if was_wrapped else key
 
 def _random_bits(key: prng.PRNGKeyArray, bit_width, shape) -> jnp.ndarray:
   key, _ = _check_prng_key(key)
   return key._random_bits(bit_width, shape)
 
 
+PRNG_IMPLS = {
+    'threefry2x32': prng.threefry_prng_impl,
+    'rbg': prng.rbg_prng_impl,
+    'unsafe_rbg': prng.unsafe_rbg_prng_impl,
+}
+
+def _get_default_prng_impl():
+  impl_name = config.jax_default_prng_impl
+  assert impl_name in PRNG_IMPLS, impl_name
+  return PRNG_IMPLS[impl_name]
+
+
 ### key operations
 
 
 def PRNGKey(seed: int) -> KeyArray:
   """Create a pseudo-random number generator (PRNG) key given an integer seed.
 
+  The resulting key carries the default PRNG implementation, as
+  determined by the ``jax_default_prng_impl`` config flag.
+
   Args:
     seed: a 64- or 32-bit integer used as the value of the key.
 
   Returns:
     A PRNG key, consumable by random functions as well as ``split``
     and ``fold_in``.
+
   """
-  return _return_prng_keys(
-      True, prng.seed_with_impl(prng.threefry_prng_impl, seed))
+  impl = _get_default_prng_impl()
+  key = prng.seed_with_impl(impl, seed)
+  return _return_prng_keys(True, key)
+
+# TODO(frostig): remove once we always enable_custom_prng
+def _check_default_impl_with_no_custom_prng(impl, name):
+  default_impl = _get_default_prng_impl()
+  default_name = config.jax_default_prng_impl
+  if not config.jax_enable_custom_prng and default_impl is not impl:
+    raise RuntimeError('jax_enable_custom_prng must be enabled in order '
+                       f'to seed an RNG with an implementation "f{name}" '
+                       f'differing from the default "f{default_name}".')
+
+def threefry2x32_key(seed: int) -> KeyArray:
+  """Creates a threefry2x32 PRNG key from an integer seed."""
+  impl = prng.threefry_prng_impl
+  _check_default_impl_with_no_custom_prng(impl, 'threefry2x32')
+  key = prng.seed_with_impl(impl, seed)
+  return _return_prng_keys(True, key)
+
+def rbg_key(seed: int) -> KeyArray:
+  """Creates an RBG PRNG key from an integer seed."""
+  impl = prng.rbg_prng_impl
+  _check_default_impl_with_no_custom_prng(impl, 'rbg')
+  key = prng.seed_with_impl(impl, seed)
+  return _return_prng_keys(True, key)
+
+def unsafe_rbg_key(seed: int) -> KeyArray:
+  """Creates an unsafe RBG PRNG key from an integer seed."""
+  impl = prng.unsafe_rbg_prng_impl
+  _check_default_impl_with_no_custom_prng(impl, 'unsafe_rbg')
+  key = prng.seed_with_impl(impl, seed)
+  return _return_prng_keys(True, key)
 
 def _fold_in(key: KeyArray, data: int) -> KeyArray:
   # Alternative to fold_in() to use within random samplers.
@@ -323,33 +371,32 @@ def shuffle(key: KeyArray, x: Array, axis: int = 0) -> jnp.ndarray:
   return _shuffle(key, x, axis)  # type: ignore
 
 
-def permutation(key: KeyArray, x: Array) -> jnp.ndarray:
-  """
-  Permute elements of an array along its first axis or return a permuted range.
+def permutation(key: KeyArray,
+                x: Union[int, Array],
+                axis: int = 0) -> jnp.ndarray:
+  """Returns a randomly permuted array or range.
 
-  If `x` is a multi-dimensional array, it is only shuffled along its
-  first index.
-
-  Args:n
+  Args:
     key: a PRNG key used as the random key.
-    x: the array or integer range to be shuffled.
+    x: int or array. If x is an integer, randomly shuffle np.arange(x).
+      If x is an array, randomly shuffle its elements.
+    axis: int, optional. The axis which x is shuffled along. Default is 0.
 
   Returns:
     A shuffled version of x or array range
   """
   key, _ = _check_prng_key(key)
+  _check_arraylike("permutation", x)
+  axis = _canonicalize_axis(axis, np.ndim(x) or 1)
   if not np.ndim(x):
-    # scalar case, must be a concrete integer
     if not np.issubdtype(lax.dtype(x), np.integer):
       raise TypeError("x must be an integer or at least 1-dimensional")
-    x = int(x)  # type: ignore[assignment]
-    return _shuffle(key, jnp.arange(x), 0)
-  elif np.ndim(x) == 1:
-    return _shuffle(key, x, 0)
-  else:
-    assert isinstance(x, jnp.ndarray)
-    ind = _shuffle(key, jnp.arange(x.shape[0]), 0)  # type: ignore[attribute-error]
-    return x[ind]
+    r = core.concrete_or_error(int, x, 'argument x of jax.random.permutation()')
+    return _shuffle(key, jnp.arange(r), axis)
+  if np.ndim(x) == 1:
+    return _shuffle(key, x, axis)
+  ind = _shuffle(key, jnp.arange(x.shape[axis]), 0)  # type: ignore[union-attr]
+  return jnp.take(x, ind, axis)
 
 
 @partial(jit, static_argnums=(2,), inline=True)
@@ -381,15 +428,16 @@ def _shuffle(key, x, axis) -> jnp.ndarray:
 
 
 def choice(key: KeyArray,
-           a: IntegerArray,
+           a: Union[int, Array],
            shape: Sequence[int] = (),
            replace: bool = True,
-           p=None) -> jnp.ndarray:
-  """Generates a random sample from a given 1-D array.
+           p: Optional[RealArray] = None,
+           axis: int = 0) -> jnp.ndarray:
+  """Generates a random sample from a given array.
 
   Args:
     key: a PRNG key used as the random key.
-    a : 1D array or int. If an ndarray, a random sample is generated from
+    a : array or int. If an ndarray, a random sample is generated from
       its elements. If an int, the random sample is generated as if a were
       arange(a).
     shape : tuple of ints, optional. Output shape.  If the given shape is,
@@ -400,6 +448,8 @@ def choice(key: KeyArray,
     p : 1-D array-like, The probabilities associated with each entry in a.
       If not given the sample assumes a uniform distribution over all
       entries in a.
+    axis: int, optional. The axis along which the selection is performed.
+      The default, 0, selects by row.
 
   Returns:
     An array of shape `shape` containing samples from `a`.
@@ -408,14 +458,13 @@ def choice(key: KeyArray,
   if not isinstance(shape, Sequence):
     raise TypeError("shape argument of jax.random.choice must be a sequence, "
                     f"got {shape}")
-  if np.ndim(a) not in [0, 1]:
-    raise ValueError("a must be an integer or 1-dimensional")
   _check_arraylike("choice", a)
   if np.ndim(a) == 0:
     a = core.concrete_or_error(int, a, "The error occurred in jax.random.choice()")
   else:
     a = jnp.asarray(a)
-  n_inputs = int(a) if np.ndim(a) == 0 else len(a)  # type: ignore[arg-type]
+  axis = _canonicalize_axis(axis, np.ndim(a) or 1)
+  n_inputs = int(a) if np.ndim(a) == 0 else a.shape[axis]  # type: ignore[arg-type]
   n_draws = prod(shape)
   if n_draws == 0:
     return jnp.zeros(shape, dtype=lax.dtype(a))
@@ -427,9 +476,10 @@ def choice(key: KeyArray,
   if p is None:
     if replace:
       ind = randint(key, shape, 0, n_inputs)
-      result = ind if np.ndim(a) == 0 else a[ind]  # type: ignore[index]
+      result = ind if np.ndim(a) == 0 else jnp.take(a, ind, axis)
     else:
-      result = permutation(key, a)[:n_draws]
+      slices = (slice(None),) * axis + (slice(n_draws),)
+      result = permutation(key, a, axis)[slices]
   else:
     if p.shape != (n_inputs,):
       raise ValueError("p must be None or match the shape of a")
@@ -437,13 +487,14 @@ def choice(key: KeyArray,
       p_cuml = jnp.cumsum(p)
       r = p_cuml[-1] * (1 - uniform(key, shape))
       ind = jnp.searchsorted(p_cuml, r)
-      result = ind if np.ndim(a) == 0 else a[ind]  # type: ignore[index]
     else:
       # Gumbel top-k trick: https://timvieira.github.io/blog/post/2019/09/16/algorithms-for-sampling-without-replacement/
       g = -gumbel(key, (n_inputs,)) - jnp.log(p)
       ind = jnp.argsort(g)[:n_draws]
-      result = ind if np.ndim(a) == 0 else a[ind]  # type: ignore[index]
-  return result.reshape(shape)
+    result = ind if np.ndim(a) == 0 else jnp.take(a, ind, axis)
+
+  return result.reshape(shape if np.ndim(a) == 0 else
+                        np.insert(np.delete(a.shape, axis), axis, shape))
 
 
 def normal(key: KeyArray,
@@ -550,10 +601,10 @@ def _multivariate_normal(key, mean, cov, shape, dtype, method) -> jnp.ndarray:
 
   if method == 'svd':
     (u, s, _) = svd(cov)
-    factor = u * jnp.sqrt(s)
+    factor = u * jnp.sqrt(s[..., None, :])
   elif method == 'eigh':
     (w, v) = eigh(cov)
-    factor = v * jnp.sqrt(w)
+    factor = v * jnp.sqrt(w[..., None, :])
   else: # 'cholesky'
     factor = cholesky(cov)
   normal_samples = normal(key, shape + mean.shape[-1:], dtype)
@@ -908,12 +959,12 @@ random_gamma_p = core.Primitive('random_gamma')
 random_gamma_p.def_impl(_gamma_impl)
 random_gamma_p.def_abstract_eval(lambda key, a: core.raise_to_shaped(a))
 ad.defjvp2(random_gamma_p, None, lambda tangent, ans, key, a: tangent * _gamma_grad(ans, a))
-xla.translations_with_avals[random_gamma_p] = xla.lower_fun(
+xla.register_translation(random_gamma_p, xla.lower_fun(
     partial(_gamma_impl, use_vmap=True),
-    multiple_results=False, with_avals=True)
-xla.backend_specific_translations['cpu'][random_gamma_p] = xla.lower_fun(
+    multiple_results=False, new_style=True))
+xla.register_translation(random_gamma_p, xla.lower_fun(
     partial(_gamma_impl, use_vmap=False),
-    multiple_results=False)
+    multiple_results=False, new_style=True), platform='cpu')
 batching.primitive_batchers[random_gamma_p] = _gamma_batching_rule
 
 def gamma(key: KeyArray,
@@ -939,8 +990,8 @@ def gamma(key: KeyArray,
   key, _ = _check_prng_key(key)
   if key.impl is not prng.threefry_prng_impl:
     raise NotImplementedError(
-        f'`gamma` is only implemented for the default PRNG, not {key.impl}')
-  return gamma_threefry2x32(key.keys, a, shape, dtype)
+        f'`gamma` is only implemented for the threefry2x32 RNG, not {key.impl}')
+  return gamma_threefry2x32(key.unsafe_raw_array(), a, shape, dtype)
 
 def gamma_threefry2x32(key: jnp.ndarray,  # raw ndarray form of a 2x32 key
                        a: RealArray,
@@ -1074,7 +1125,8 @@ def poisson(key: KeyArray,
   key, _ = _check_prng_key(key)
   if key.impl is not prng.threefry_prng_impl:
     raise NotImplementedError(
-        f'`poisson` is only implemented for the default PRNG, not {key.impl}')
+        '`poisson` is only implemented for the threefry2x32 RNG, '
+        f'not {key.impl}')
   dtype = dtypes.canonicalize_dtype(dtype)
   shape = core.canonicalize_shape(shape)
   if np.shape(lam) != shape:
