@@ -12,15 +12,16 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import functools
 import collections
+import functools
+from functools import partial
 import operator as op
 from typing import (Any, Callable, Hashable, Iterable, Optional, Tuple, Type,
                     TypeVar, overload, TYPE_CHECKING)
 
-from ..lib import pytree
+from jax._src.lib import pytree
 
-from .._src.util import partial, safe_zip, unzip2
+from .._src.util import safe_zip, unzip2
 
 from .._src import traceback_util
 traceback_util.register_exclusion(__file__)
@@ -187,6 +188,9 @@ def build_tree(treedef, xs):
   return treedef.from_iterable_tree(xs)
 
 def tree_transpose(outer_treedef, inner_treedef, pytree_to_transpose):
+  """Transform a tree having tree structure (outer, inner) into one having structure
+  (inner, outer).
+  """
   flat, treedef = tree_flatten(pytree_to_transpose)
   inner_size = inner_treedef.num_leaves
   outer_size = outer_treedef.num_leaves
@@ -311,6 +315,20 @@ class Partial(functools.partial):
   >>> call_func(print_zero)
   Traced<ShapedArray(int32[], weak_type=True)>with<DynamicJaxprTrace(level=0/1)>
   """
+  def __new__(klass, func, *args, **kw):
+    # In Python 3.10+, if func is itself a functools.partial instance,
+    # functools.partial.__new__ would merge the arguments of this Partial
+    # instance with the arguments of the func. We box func in another lambda to
+    # avoid this optimization since it would change which arguments are
+    # considered part of the pytree.
+    if isinstance(func, functools.partial):
+      original_func = func
+      func = lambda *args, **kw: original_func(*args, **kw)
+      func.func = original_func.func
+      func.args = original_func.args
+      func.keywords = original_func.keywords
+    return super(Partial, klass).__new__(klass, func, *args, **kw)
+
 
 register_pytree_node(
     Partial,
