@@ -456,15 +456,15 @@ from jax import custom_derivatives
 from jax._src import dtypes
 from jax import lax
 from jax.experimental import pjit
-from jax.lib import pytree
-from jax.lib import xla_bridge as xb
-from jax.lib import xla_client
-from jax.lib import xla_extension
 from jax.interpreters import ad, xla, batching, masking, pxla
 from jax.interpreters import partial_eval as pe
-from jax._src import pprint_util as ppu
+from jax._src import pretty_printer as pp
 from jax._src import source_info_util
 from jax._src import util
+from jax._src.lib import pytree
+from jax._src.lib import xla_bridge as xb
+from jax._src.lib import xla_client
+from jax._src.lib import xla_extension
 
 import numpy as np
 
@@ -496,7 +496,7 @@ xops = xla_client._xla.ops
 
 XlaOp = xla_client.XlaOp
 XlaShape = xla_client.Shape
-XlaComputationBuilder = xla_client.XlaBuilder
+XlaBuilder = xla_client.XlaBuilder
 XlaDevice = xla_client.Device
 XlaLocalClient = xla_client.Client
 DType = Any
@@ -747,21 +747,31 @@ def _print_tap_func(
       f"{k}: {v}" for k, v in sorted(kwargs.items())
   ])
 
-  def pp_val(arg) -> ppu.PrettyPrint:
+  def pp_val(arg) -> pp.Doc:
     if isinstance(arg, tuple):
-      return (
-          ppu.pp("( ") >> ppu.vcat([pp_val(e) for e in arg]) >> ppu.pp(" )"))
+      return pp.group(pp.concat([
+        pp.text("( "),
+        pp.nest(2, pp.join(pp.brk(), [pp_val(e) for e in arg])),
+        pp.text(" )")
+      ]))
     elif isinstance(arg, list):
-      return (
-          ppu.pp("[ ") >> ppu.vcat([pp_val(e) for e in arg]) >> ppu.pp(" ]"))
+      return pp.group(pp.concat([
+        pp.text("[ "),
+        pp.nest(2, pp.join(pp.brk(), [pp_val(e) for e in arg])),
+        pp.text(" ]")
+      ]))
     elif isinstance(arg, dict):
-      return (ppu.pp("{ ") >> ppu.vcat([
-          ppu.pp(f"{k}=") >> pp_val(v) for k, v in sorted(arg.items())
-      ]) >> ppu.pp(" }"))
+      return pp.group(pp.concat([
+        pp.text("{ "),
+        pp.nest(2, pp.join(pp.brk(), [
+          pp.text(f"{k}=") + pp_val(v) for k, v in sorted(arg.items())
+        ])),
+        pp.text(" }")
+      ]))
     elif isinstance(arg, np.ndarray):
-      return ppu.pp(np.array2string(arg, threshold=threshold))
+      return pp.text(np.array2string(arg, threshold=threshold))
     else:
-      return ppu.pp(str(arg))
+      return pp.text(str(arg))
 
   with _print_tap_lock:
     if kv_pairs:
@@ -773,20 +783,18 @@ def _values_to_avals(vals) -> Sequence[core.ShapedArray]:
   return tuple([core.raise_to_shaped(core.get_aval(v)) for v in vals])
 
 ### The id_tap_dep primitive
-"""
-The id_tap_dep_p primitive is used to create a dependency of the result of
-id_tap on the actual tap operation. This is only needed when the
-id_tap function is used with the `result` parameter. This primitive acts
-as the identity operator on the first argument.
-
-For example, given `id_tap(f, (a, b), result=(r, s)`, we convert this to
-
-   a1, b1 = outside_call_p(f, a, b)
-   r1 = id_tap_dep_p(r, a1)
-   s1 = id_tap_dep_p(s, a1)
-
-There are always two arguments and the result is equal to the first.
-"""
+# The id_tap_dep_p primitive is used to create a dependency of the result of
+# id_tap on the actual tap operation. This is only needed when the
+# id_tap function is used with the `result` parameter. This primitive acts
+# as the identity operator on the first argument.
+#
+# For example, given `id_tap(f, (a, b), result=(r, s)`, we convert this to
+#
+#    a1, b1 = outside_call_p(f, a, b)
+#    r1 = id_tap_dep_p(r, a1)
+#    s1 = id_tap_dep_p(s, a1)
+#
+# There are always two arguments and the result is equal to the first.
 id_tap_dep_p = core.Primitive("id_tap_dep")
 id_tap_dep_p.multiple_results = False
 id_tap_dep_p.def_impl(lambda r, _: r)
@@ -915,7 +923,7 @@ def _outside_call_impl(*args, **params):
 outside_call_p.def_impl(_outside_call_impl)
 
 
-def _outside_call_translation_rule(comp: XlaComputationBuilder,
+def _outside_call_translation_rule(comp: XlaBuilder,
                                    *args_op: XlaOp,
                                    platform="tpu",
                                    has_token,
@@ -1136,7 +1144,7 @@ def _outside_call_run_callback(
       return canonical_flat_results
 
   except Exception as e:
-    logging.error(f"Outside call {callback} threw exception {e}.")
+    logging.error("Outside call %s threw exception %s.", callback, e)
     if send_infeed:
       # Prepare some results to send in case of error. We are sending something
       # with a distinctive shape (int8[12345]), one that is unlikely to be what the device
@@ -1215,28 +1223,31 @@ ad.primitive_jvps[outside_call_p] = _outside_call_jvp_rule
 
 
 def _outside_call_partial_eval_rule(trace, *args, **params):
-  # The args have been prepared by the id_tap_jvp_rule: primals, tangents
+  # partial eval is used after jvp and before transpose.
   transforms = params.get("transforms", ())
   if not transforms or transforms[-1] != ("jvp",):
     # We are not in the process of computing VJP
     return trace.default_process_primitive(outside_call_p, args, params)
 
+  # The args have been prepared by the id_tap_jvp_rule: primals, tangents. The
+  # result is a pair of the primal outputs and output tangents.
+  # One invariant that JAX requires is that if the primals arguments are known
+  # then the primal outputs must be known. So, if the primal arguments are known
+  # and some of the tangents are unknown, then we must split the tap into
+  # one for the primals (thus the output will be considered known), and a
+  # separate tap for the tangents.
   assert "has_token" not in params
   if not params["identity"]:
     raise NotImplementedError("differentiation rules are implemented only for id_tap, not for call.")
 
   assert len(args) % 2 == 0
   nr_primals = len(args) // 2
-
-  consts = [t.pval.get_known() for t in args]
-  if all(c is not None for c in consts):
-    return trace.default_process_primitive(outside_call_p, args, params)
-  # Split into two taps, one for the knowns and one for the unknowns
-  # We implement here only the case when primals are known, and we make a tap
-  # with just the primals.
   primals, tangents = util.split_list(args, [nr_primals])
-  c_primals_tapped, _ = util.split_list(consts, [nr_primals])
-  assert all([c is not None for c in c_primals_tapped])
+  all_primals_known = all(p.is_known() for p in primals)
+  some_tangents_unknown = any(not t.is_known() for t in tangents)
+
+  if not (all_primals_known and some_tangents_unknown):
+    return trace.default_process_primitive(outside_call_p, args, params)
 
   prims, _ = params["arg_treedef"].unflatten(args)
   _, primals_treedef = api.tree_flatten(prims)
@@ -1288,7 +1299,7 @@ def _outside_call_transpose_rule(cts, *args, **params):
       *cts_tangents,
       **dict(_add_transform(params, "transpose"),
              arg_treedef=vjp_arg_treedef))
-  return (cts_primals + cts_tangents_through_tap)
+  return cts_primals + cts_tangents_through_tap
 
 
 ad.primitive_transposes[outside_call_p] = _outside_call_transpose_rule
