@@ -58,7 +58,7 @@ def interp_fit_dopri(y0, y1, k, dt):
   dps_c_mid = jnp.array([
       6025192743 / 30085553152 / 2, 0, 51252292925 / 65400821598 / 2,
       -2691868925 / 45128329728 / 2, 187940372067 / 1594534317056 / 2,
-      -1776094331 / 19743644256 / 2, 11237099 / 235043384 / 2])
+      -1776094331 / 19743644256 / 2, 11237099 / 235043384 / 2], dtype=y0.dtype)
   y_mid = y0 + dt * jnp.dot(dps_c_mid, k)
   return jnp.asarray(fit_4th_order_polynomial(y0, y1, y_mid, k[0], k[-1], dt))
 
@@ -92,19 +92,21 @@ def initial_step_size(fun, t0, y0, order, rtol, atol, f0):
 
 def runge_kutta_step(func, y0, f0, t0, dt):
   # Dopri5 Butcher tableaux
-  alpha = jnp.array([1 / 5, 3 / 10, 4 / 5, 8 / 9, 1., 1., 0])
-  beta = jnp.array([
-      [1 / 5, 0, 0, 0, 0, 0, 0],
-      [3 / 40, 9 / 40, 0, 0, 0, 0, 0],
-      [44 / 45, -56 / 15, 32 / 9, 0, 0, 0, 0],
-      [19372 / 6561, -25360 / 2187, 64448 / 6561, -212 / 729, 0, 0, 0],
-      [9017 / 3168, -355 / 33, 46732 / 5247, 49 / 176, -5103 / 18656, 0, 0],
-      [35 / 384, 0, 500 / 1113, 125 / 192, -2187 / 6784, 11 / 84, 0]
-  ])
-  c_sol = jnp.array([35 / 384, 0, 500 / 1113, 125 / 192, -2187 / 6784, 11 / 84, 0])
-  c_error = jnp.array([35 / 384 - 1951 / 21600, 0, 500 / 1113 - 22642 / 50085,
-                      125 / 192 - 451 / 720, -2187 / 6784 - -12231 / 42400,
-                      11 / 84 - 649 / 6300, -1. / 60.])
+  alpha = jnp.array([1 / 5, 3 / 10, 4 / 5, 8 / 9, 1., 1., 0], dtype=f0.dtype)
+  beta = jnp.array(
+      [[1 / 5, 0, 0, 0, 0, 0, 0], [3 / 40, 9 / 40, 0, 0, 0, 0, 0],
+       [44 / 45, -56 / 15, 32 / 9, 0, 0, 0, 0],
+       [19372 / 6561, -25360 / 2187, 64448 / 6561, -212 / 729, 0, 0, 0],
+       [9017 / 3168, -355 / 33, 46732 / 5247, 49 / 176, -5103 / 18656, 0, 0],
+       [35 / 384, 0, 500 / 1113, 125 / 192, -2187 / 6784, 11 / 84, 0]],
+      dtype=f0.dtype)
+  c_sol = jnp.array(
+      [35 / 384, 0, 500 / 1113, 125 / 192, -2187 / 6784, 11 / 84, 0],
+      dtype=f0.dtype)
+  c_error = jnp.array([
+      35 / 384 - 1951 / 21600, 0, 500 / 1113 - 22642 / 50085, 125 / 192 -
+      451 / 720, -2187 / 6784 - -12231 / 42400, 11 / 84 - 649 / 6300, -1. / 60.
+  ], dtype=f0.dtype)
 
   def body_fun(i, k):
     ti = t0 + dt * alpha[i-1]
