@@ -91,7 +91,7 @@ BCOO objects are designed to be compatible with JAX transforms, including :func:
 
     >>> from jax import grad, jit
 
-    >>> def f(y): 
+    >>> def f(y):
     ...   return (M_sp.T @ y).sum()
     ...
     >>> jit(grad(f))(y)
@@ -141,6 +141,7 @@ Example: sparse logistic regression
 As an example of a more complicated sparse workflow, let's consider a simple logistic regression
 implemented in JAX. Notice that the following implementation has no reference to sparsity:
 
+    >>> import functools
     >>> from sklearn.datasets import make_classification
     >>> from jax.scipy import optimize
 
@@ -156,7 +157,8 @@ implemented in JAX. Notice that the following implementation has no reference to
     ...
     >>> def fit_logreg(X, y):
     ...   params = jnp.zeros(X.shape[1] + 1)
-    ...   result = optimize.minimize(jax.partial(loss, X=X, y=y), x0=params, method='BFGS')
+    ...   result = optimize.minimize(functools.partial(loss, X=X, y=y),
+    ...                              x0=params, method='BFGS')
     ...   return result.x
 
     >>> X, y = make_classification(n_classes=2, random_state=1701)
@@ -181,42 +183,50 @@ To fit the same model on sparse data, we can apply the :func:`sparsify` transfor
 """
 
 # flake8: noqa: F401
-from .ad import grad, value_and_grad
-from .ops import (
-    bcoo_dot_general,
-    bcoo_dot_general_p,
-    bcoo_dot_general_sampled,
-    bcoo_dot_general_sampled_p,
-    bcoo_extract,
-    bcoo_extract_p,
-    bcoo_fromdense,
-    bcoo_fromdense_p,
-    bcoo_reduce_sum,
-    bcoo_rdot_general,
-    bcoo_todense,
-    bcoo_todense_p,
-    bcoo_transpose,
-    bcoo_transpose_p,
-    coo_fromdense,
-    coo_fromdense_p,
-    coo_matmat,
-    coo_matmat_p,
-    coo_matvec,
-    coo_matvec_p,
-    coo_todense,
-    coo_todense_p,
-    csr_fromdense,
-    csr_fromdense_p,
-    csr_matmat,
-    csr_matmat_p,
-    csr_matvec,
-    csr_matvec_p,
-    csr_todense,
-    csr_todense_p,
-    COO,
-    CSC,
-    CSR,
-    BCOO,
+from .ad import (
+    grad as grad,
+    value_and_grad as value_and_grad,
+)
+from .bcoo import (
+    bcoo_dot_general as bcoo_dot_general,
+    bcoo_dot_general_p as bcoo_dot_general_p,
+    bcoo_dot_general_sampled as bcoo_dot_general_sampled,
+    bcoo_dot_general_sampled_p as bcoo_dot_general_sampled_p,
+    bcoo_extract as bcoo_extract,
+    bcoo_extract_p as bcoo_extract_p,
+    bcoo_fromdense as bcoo_fromdense,
+    bcoo_fromdense_p as bcoo_fromdense_p,
+    bcoo_reduce_sum as bcoo_reduce_sum,
+    bcoo_rdot_general as bcoo_rdot_general,
+    bcoo_spdot_general as bcoo_spdot_general,
+    bcoo_spdot_general_p as bcoo_spdot_general_p,
+    bcoo_todense as bcoo_todense,
+    bcoo_todense_p as bcoo_todense_p,
+    bcoo_transpose as bcoo_transpose,
+    bcoo_transpose_p as bcoo_transpose_p,
+    BCOO as BCOO,
 )
 
-from .transform import sparsify
\ No newline at end of file
+from .ops import (
+    coo_fromdense as coo_fromdense,
+    coo_fromdense_p as coo_fromdense_p,
+    coo_matmat as coo_matmat,
+    coo_matmat_p as coo_matmat_p,
+    coo_matvec as coo_matvec,
+    coo_matvec_p as coo_matvec_p,
+    coo_todense as coo_todense,
+    coo_todense_p as coo_todense_p,
+    csr_fromdense as csr_fromdense,
+    csr_fromdense_p as csr_fromdense_p,
+    csr_matmat as csr_matmat,
+    csr_matmat_p as csr_matmat_p,
+    csr_matvec as csr_matvec,
+    csr_matvec_p as csr_matvec_p,
+    csr_todense as csr_todense,
+    csr_todense_p as csr_todense_p,
+    COO as COO,
+    CSC as CSC,
+    CSR as CSR,
+)
+
+from .transform import sparsify as sparsify
