@@ -55,7 +55,8 @@ import numpy as np
 from jax import core
 from jax import lax
 from jax import linear_util as lu
-from jax.api_util import flatten_fun_nokwargs
+import jax.numpy as jnp
+from jax._src.api_util import flatten_fun_nokwargs
 from jax.interpreters import partial_eval as pe
 from jax.interpreters import xla
 from jax.tree_util import tree_flatten, tree_map, tree_unflatten
@@ -78,7 +79,7 @@ class SparseEnv:
     self._buffers = list(bufs)
 
   def push(self, arr: Array) -> int:
-    self._buffers.append(np.array(arr) if np.isscalar(arr) else arr)  # type: ignore
+    self._buffers.append(jnp.asarray(arr))  # type: ignore
     return len(self._buffers) - 1
 
   def get(self, ind: int) -> Array:
@@ -155,7 +156,8 @@ def argspecs_to_avals(
     if argspec.is_unit():
       return core.abstract_unit
     else:
-      return core.ShapedArray(argspec.shape, argspec.data(spenv).dtype)
+      data = argspec.data(spenv)
+      return core.ShapedArray(argspec.shape, data.dtype, data.aval.weak_type)
   return tree_map(argspec_to_aval, argspecs, is_leaf=_is_argspec)
 
 
@@ -175,13 +177,14 @@ def eval_sparse(
       return env[var]
 
   def write_buffer(var: core.Var, a: Array) -> None:
-    if var is core.dropvar:
+    if isinstance(var, core.DropVar):
       return
     env[var] = ArgSpec(a.shape, spenv.push(a), None)
 
   def write(var: core.Var, a: ArgSpec) -> None:
-    if var is core.dropvar:
+    if isinstance(var, core.DropVar):
       return
+    assert a is not None
     env[var] = a
 
   # TODO: handle unitvar at all?
@@ -208,8 +211,11 @@ def eval_sparse(
         out_bufs = prim.bind(*(val.data(spenv) for val in invals), **eqn.params)
       out_bufs = out_bufs if prim.multiple_results else [out_bufs]
       out = []
-      for buf in out_bufs:
-        out.append(ArgSpec(buf.shape, spenv.push(buf), None))
+      for buf, outvar in safe_zip(out_bufs, eqn.outvars):
+        if isinstance(outvar, core.DropVar):
+          out.append(None)
+        else:
+          out.append(ArgSpec(buf.shape, spenv.push(buf), None))
     safe_map(write, eqn.outvars, out)
 
   return safe_map(read, jaxpr.outvars)
@@ -276,10 +282,14 @@ for _prim in [
   sparse_rules[_prim] = _zero_preserving_unary_op(_prim)
 
 def _dot_general_sparse(spenv, *argspecs, dimension_numbers, precision, preferred_element_type):
-  if argspecs[0].is_sparse() and argspecs[1].is_sparse():
-    raise NotImplementedError("dot_general between two sparse matrices.")
   A, B = argspecs_to_arrays(spenv, argspecs)
-  if argspecs[0].is_sparse():
+  if argspecs[0].is_sparse() and argspecs[1].is_sparse():
+    shape = sparse.bcoo._dot_general_validated_shape(A.shape, B.shape, dimension_numbers)
+    data, indices = sparse.bcoo_spdot_general(A.data, A.indices, B.data, B.indices,
+                                              lhs_shape=A.shape, rhs_shape=B.shape,
+                                              dimension_numbers=dimension_numbers)
+    return [ArgSpec(shape, spenv.push(data), spenv.push(indices))]
+  elif argspecs[0].is_sparse():
     result = sparse.bcoo_dot_general(A.data, A.indices, B, lhs_shape=A.shape,
                                     dimension_numbers=dimension_numbers)
   else:
@@ -414,6 +424,10 @@ def _sparsify_jaxpr(spenv, jaxpr, *argspecs):
 
   @lu.wrap_init
   def wrapped(*args_flat):
+    # TODO(frostig,jakevdp): This closes over `spenv`, which can bring
+    # in buffers from the "outer scope" as constants. Is this a
+    # problem for primitives like cond and while_loop, which always
+    # convert constvars to invars when staging out their subjaxprs?
     nonlocal out_tree
     args = tree_unflatten(in_tree, args_flat)
     argspecs = arrays_to_argspecs(spenv, args)
@@ -426,7 +440,7 @@ def _sparsify_jaxpr(spenv, jaxpr, *argspecs):
   args_flat, in_tree = tree_flatten(args)
   avals_flat = [core.raise_to_shaped(core.get_aval(arg)) for arg in args_flat]
   sp_jaxpr, _, consts = pe.trace_to_jaxpr_dynamic(wrapped, avals_flat)
-  sp_jaxpr = pe.ClosedJaxpr(pe.convert_constvars_jaxpr(sp_jaxpr), consts)
+  sp_jaxpr = pe.ClosedJaxpr(sp_jaxpr, consts)
   return sp_jaxpr, out_tree
 
 def _while_sparse(spenv, *argspecs, cond_jaxpr, cond_nconsts, body_jaxpr, body_nconsts):
