@@ -14,6 +14,7 @@
 
 
 import functools
+from functools import partial
 import itertools as it
 from typing import Any, Callable, Dict
 
@@ -26,11 +27,11 @@ from ..core import (Trace, Tracer, get_aval, call_p, Primitive, Literal,
                     raise_to_shaped)
 from .._src.ad_util import (add_jaxvals, add_jaxvals_p, zeros_like_jaxval,
                               zeros_like_aval, zeros_like_p, Zero)
-from .._src.util import (unzip2, safe_map, safe_zip, partial, split_list,
+from .._src.util import (unzip2, safe_map, safe_zip, split_list,
                          wrap_name, as_hashable_function)
 from ..tree_util import register_pytree_node
 from .. import linear_util as lu
-from ..api_util import flatten_fun, flatten_fun_nokwargs
+from .._src.api_util import flatten_fun, flatten_fun_nokwargs
 from ..tree_util import tree_flatten, tree_unflatten, Partial
 from .._src import source_info_util
 
@@ -233,6 +234,10 @@ def backward_pass(jaxpr: core.Jaxpr, reduce_axes, consts, primals_in, cotangents
   cotangents_out = map(read_cotangent, jaxpr.invars)
   return cotangents_out
 
+def closed_backward_pass(jaxpr: core.ClosedJaxpr, reduce_axes, primals_in, cotangents_in):
+  return backward_pass(jaxpr.jaxpr, reduce_axes, jaxpr.consts, primals_in, cotangents_in)
+
+
 class UndefinedPrimal:
   __slots__ = ['aval']
   def __init__(self, aval):
@@ -365,7 +370,7 @@ class JVPTrace(Trace):
     avals_out = [raise_to_shaped(core.get_aval(x)) for x in primals_out]
     tangents_out = custom_lin_p.bind(
         *res, *tangents_in, num_res=res_tree.num_leaves, bwd=bwd,
-        avals_out=avals_out)
+        out_avals=avals_out)
     tangents_out = map(recast_to_float0, primals_out, tangents_out)
     return map(partial(JVPTracer, self), primals_out, tangents_out)
 
@@ -622,7 +627,7 @@ def map_transpose(primitive, params, call_jaxpr, args, ct, _, reduce_axes):
   assert len(in_axes) == len(arg_cts)
   def unmap_zero(zero, in_axis):
     return (zero if in_axis is None else
-            Zero(core.unmapped_aval(params['axis_size'], in_axis, zero.aval)))
+            Zero(core.unmapped_aval(params['axis_size'], params['axis_name'], in_axis, zero.aval)))
   arg_cts = (unmap_zero(arg_ct, in_axis) if type(arg_ct) is Zero else
              arg_ct if in_axis is not None else
              arg_ct.sum(0)
@@ -671,7 +676,7 @@ def _interleave(xs, ys):
 
 
 custom_lin_p = core.Primitive('custom_lin')
-custom_lin_p.def_abstract_eval(lambda *_, avals_out, **__: avals_out)
+custom_lin_p.def_abstract_eval(lambda *_, out_avals, **__: out_avals)
 custom_lin_p.multiple_results = True
 
 def _raise_custom_vjp_error_on_jvp(*_, **__):
@@ -679,9 +684,9 @@ def _raise_custom_vjp_error_on_jvp(*_, **__):
                   "function.")
 custom_lin_p.def_impl(_raise_custom_vjp_error_on_jvp)
 
-def _custom_lin_transpose(cts_out, *invals, num_res, bwd, avals_out):
+def _custom_lin_transpose(cts_out, *invals, num_res, bwd, out_avals):
   res, _ = split_list(invals, [num_res])
-  cts_out = map(instantiate_zeros_aval, avals_out, cts_out)
+  cts_out = map(instantiate_zeros_aval, out_avals, cts_out)
   cts_in = bwd.call_wrapped(*res, *cts_out)
   return [None] * num_res + list(cts_in)
 primitive_transposes[custom_lin_p] = _custom_lin_transpose
