@@ -12,11 +12,13 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import inspect
-import itertools as it
 from collections import namedtuple
 import contextlib
 import functools
+from functools import partial
+import inspect
+import itertools as it
+import operator as op
 from typing import (Any, Callable, Dict, NamedTuple, Optional, Sequence, Tuple,
                     List, Union, cast)
 from weakref import ref
@@ -27,14 +29,14 @@ from .. import core
 from .._src import dtypes
 from .. import linear_util as lu
 from jax._src.ad_util import Zero
-from ..api_util import flattened_fun_in_tree
+from .._src.api_util import flattened_fun_in_tree
 from .._src.tree_util import PyTreeDef, tree_unflatten, tree_leaves
-from .._src.util import (unzip2, safe_zip, safe_map, toposort, partial,
-                         split_list, partition_list, cache, OrderedSet,
+from .._src.util import (unzip2, safe_zip, safe_map, toposort, split_list,
+                         partition_list, cache, OrderedSet,
                          as_hashable_function)
 from ..core import (Trace, Tracer, Jaxpr, Literal, get_aval, AbstractValue,
                     unit, unitvar, abstract_unit, ClosedJaxpr, new_jaxpr_eqn,
-                    dropvar, ConcreteArray, raise_to_shaped, Var, Atom,
+                    ConcreteArray, raise_to_shaped, Var, Atom,
                     JaxprEqn, Primitive)
 from jax._src import source_info_util
 from ..config import config
@@ -115,6 +117,10 @@ class JaxprTrace(Trace):
 
   def new_arg(self, pval: PartialVal) -> 'JaxprTracer':
     const = pval.get_known()
+    # XXX: Think twice before changing this constant argument pruning!
+    # This has really important consequences for partial_eval_jaxpr.
+    # Most importantly, this guarantees that the unknown jaxpr never uses
+    # known inputs (if it needs them, then they get passed through residuals).
     if const is None:
       return JaxprTracer(self, pval, LambdaBinding())
     else:
@@ -146,7 +152,7 @@ class JaxprTrace(Trace):
 
   def default_process_primitive(self, primitive, tracers, params):
     """By default, if all the input tracers are known, then execute the primitive
-    and all the ouputs are known. Otherwise, all the outputs are unknown."""
+    and all the outputs are known. Otherwise, all the outputs are unknown."""
     consts = [t.pval.get_known() for t in tracers]
     if all(c is not None for c in consts):
       return primitive.bind(*consts, **params)
@@ -196,7 +202,7 @@ class JaxprTrace(Trace):
       jaxpr, out_pvals, consts, env_tracers = self.partial_eval(
           f, in_pvals, app, instantiate=False)
       if primitive.map_primitive:
-        unmapped_aval = partial(core.unmapped_aval, params['axis_size'])
+        unmapped_aval = partial(core.unmapped_aval, params['axis_size'], params['axis_name'])
         out_axes = params['out_axes_thunk']()
         out_pvals = [pval if pval.is_known() else
                      PartialVal.unknown(unmapped_aval(out_axis, pval[0])) if out_axis is not None else
@@ -206,7 +212,7 @@ class JaxprTrace(Trace):
       # Skip known invars and outvars, and lift constants as regular invars
       in_knowns = tuple(t.pval.is_known() for t in it.chain(env_tracers, tracers))
       out_unknowns = tuple(not pval.is_known() for pval in out_pvals)
-      jaxpr = _drop_invars(jaxpr, in_knowns)
+      jaxpr = _drop_vars(jaxpr, in_knowns, (False,) * len(jaxpr.outvars))
       jaxpr = _dce_open_jaxpr(jaxpr, out_unknowns, drop_outputs=True)
 
       # Known tracers get propagated as if they were constants
@@ -258,7 +264,7 @@ class JaxprTrace(Trace):
     if primitive.map_primitive:
       out_axes = params['out_axes_thunk']()
       sz = params['axis_size']
-      out_pvs = [None if pv is None else core.unmapped_aval(sz, ax, pv)
+      out_pvs = [None if pv is None else core.unmapped_aval(sz, params['axis_name'], ax, pv)
                  for pv, ax in zip(out_pvs, out_axes)]
 
     def todo(x):
@@ -581,8 +587,8 @@ def recipe_to_eqn(getvar: Callable[[JaxprTracer], Atom],
   _, in_tracers, out_tracer_refs, primitive, params, source_info = recipe
   out_tracers = [t_ref() for t_ref in out_tracer_refs]
   invars  = [getvar(t) for t in in_tracers]
-  outvars = [core.dropvar if t is None else cast(Var, getvar(t))
-             for t in out_tracers]
+  outvars = [core.DropVar(core.abstract_unit) if t is None
+             else cast(Var, getvar(t)) for t in out_tracers]
   return new_jaxpr_eqn(invars, outvars, primitive, params, source_info)
 
 def tracers_to_jaxpr(
@@ -783,7 +789,7 @@ def _remat_partial_eval(trace, _, f, tracers, params):
   in_unknowns = ([False] * len(consts) +
                  [not t.is_known() for t in it.chain(env_tracers, tracers)])
   if params['policy']:
-    # unzip into known and jaxpr_unknown
+    # unzip into jaxpr_known and jaxpr_unknown
     jaxpr_known, jaxpr_unknown, out_unknowns, out_inst, _ = _partial_eval_jaxpr_custom(
         jaxpr, in_unknowns, params['policy'])
     jaxpr_known, in_used_known = dce_jaxpr(jaxpr_known, [True] * len(jaxpr_known.outvars))
@@ -965,14 +971,18 @@ def partial_eval_jaxpr_custom_rule_not_implemented(
 ParamsUpdater = Callable[[List[bool], int, dict, dict], Tuple[dict, dict]]
 
 def call_partial_eval_custom_rule(
-    params_updater: ParamsUpdater, saveable: Callable[..., bool],
-    unks_in: List[bool], inst_in: List[bool], eqn: JaxprEqn
+    jaxpr_param_name: str, params_updater: ParamsUpdater,
+    saveable: Callable[..., bool], unks_in: List[bool], inst_in: List[bool],
+    eqn: JaxprEqn
   ) -> Tuple[JaxprEqn, JaxprEqn, Sequence[bool], Sequence[bool], List[Var]]:
-  jaxpr_known, jaxpr_staged, unks_out, inst_out, num_res = _partial_eval_jaxpr_custom(
-      eqn.params['call_jaxpr'], unks_in, saveable)
+  jaxpr = eqn.params[jaxpr_param_name]
+  jaxpr_known, jaxpr_staged, unks_out, inst_out, num_res = \
+      _partial_eval_jaxpr_custom(jaxpr, unks_in, saveable)
   ins_known, _ = partition_list(unks_in, eqn.invars)
-  out_binders_known, _ = partition_list(unks_out, eqn.outvars)
-  out_binders_known = [v for v in out_binders_known if v is not dropvar]
+  # by convention, _partial_eval_jaxpr_custom drops units on known outputs
+  known_units_out = [v.aval is core.abstract_unit for v in jaxpr.outvars]
+  dropped_outs_known = map(op.or_, unks_out, known_units_out)
+  out_binders_known, _ = partition_list(dropped_outs_known, eqn.outvars)
   _, out_binders_staged = partition_list(inst_out, eqn.outvars)
   newvar = core.gensym([jaxpr_known, jaxpr_staged])
   residuals = [newvar(v.aval) for v in jaxpr_staged.invars[:num_res]]
@@ -988,11 +998,13 @@ def call_partial_eval_custom_rule(
               if type(x) is Var and not inst]
   return eqn_known, eqn_staged, unks_out, inst_out, new_inst + residuals
 partial_eval_jaxpr_custom_rules[core.call_p] = \
-    partial(call_partial_eval_custom_rule, lambda _, __, x, y: (x, y))
+    partial(call_partial_eval_custom_rule, 'call_jaxpr',
+            lambda _, __, x, y: (x, y))
 partial_eval_jaxpr_custom_rules[core.named_call_p] = \
-    partial(call_partial_eval_custom_rule, lambda _, __, x, y: (x, y))
+    partial(call_partial_eval_custom_rule, 'call_jaxpr',
+            lambda _, __, x, y: (x, y))
 partial_eval_jaxpr_custom_rules[remat_call_p] = \
-    partial(call_partial_eval_custom_rule,
+    partial(call_partial_eval_custom_rule, 'call_jaxpr',
             lambda _, __, p1, p2: (p1, dict(p2, differentiated=True)))
 
 
@@ -1013,13 +1025,18 @@ def dce_jaxpr(jaxpr: Jaxpr, used_outputs: List[bool]
   map(write, jaxpr.outvars, used_outputs)
   for eqn in jaxpr.eqns[::-1]:
     used_outs = map(read, eqn.outvars)
-    rule = dce_rules.get(eqn.primitive)
-    if rule:
-      used_ins, new_eqn = rule(used_outs, eqn)
-      if any(used_ins): new_eqns.append(new_eqn)
-    elif any(used_outs):
-      new_eqns.append(eqn)
-      used_ins = [True] * len(eqn.invars)
+    # If any outputs are used, then we need to keep a version of the eqn and
+    # potentially mark some inputs as used. Otherwise mark all inputs as unused.
+    if any(used_outs):
+      # If there's a rule for modifying the eqn and computing used inputs, apply
+      # it. Otherwise, keep the eqn unmodified and mark all inputs as used.
+      rule = dce_rules.get(eqn.primitive)
+      if rule:
+        used_ins, new_eqn = rule(used_outs, eqn)
+      else:
+        used_ins = [True] * len(eqn.invars)
+        new_eqn = eqn
+      new_eqns.append(new_eqn)
     else:
       used_ins = [False] * len(eqn.invars)
     map(write, eqn.invars, used_ins)
@@ -1078,9 +1095,11 @@ def _dce_open_jaxpr(jaxpr: Jaxpr, outputs: Tuple[bool, ...], drop_outputs=False)
   return Jaxpr(jaxpr.constvars, jaxpr.invars, new_outvars, new_eqns)
 
 @cache()
-def _drop_invars(jaxpr: Jaxpr, drop: Tuple[bool, ...]):
-  return Jaxpr(jaxpr.constvars, [v for v, d in zip(jaxpr.invars, drop) if not d],
-               jaxpr.outvars, jaxpr.eqns)
+def _drop_vars(jaxpr: Jaxpr, drop_ins: Tuple[bool, ...], drop_outs: Tuple[bool, ...]):
+  return Jaxpr(jaxpr.constvars,
+               [v for v, d in zip(jaxpr.invars, drop_ins) if not d],
+               [v for v, d in zip(jaxpr.outvars, drop_outs) if not d],
+               jaxpr.eqns)
 
 
 def _reconstruct_pval(pval1: PartialVal, const2: core.Value):
@@ -1135,7 +1154,8 @@ class DynamicJaxprTracer(core.Tracer):
                 f"depends on the value{'s' if len(invar_pos) > 1 else ''} "
                 f"of {dbg.arg_info(invar_pos)}.")
     elif progenitor_eqns:
-      msts = [f"  operation {core.pp_eqn(eqn, print_shapes=True)}\n"
+      msts = ["  operation "
+              f"{core.pp_eqn(eqn, core.JaxprPpContext(), print_shapes=True)}\n"
               f"    from line {source_info_util.summarize(eqn.source_info)}"
               for eqn in progenitor_eqns]
       origin = (f"While tracing the function {dbg.func_src_info} "
@@ -1169,6 +1189,7 @@ class JaxprStackFrame:
     outvars = [self.tracer_to_var[id(t)] for t in out_tracers]
     constvars, constvals = unzip2(self.constvar_to_val.items())
     jaxpr = Jaxpr(constvars, invars, outvars, self.eqns)
+    jaxpr, constvals = _prune_convert_element_types(jaxpr, constvals)
     jaxpr, constvals = _inline_literals(jaxpr, constvals)
     out_avals = [t.aval for t in out_tracers]
     return jaxpr, out_avals, constvals
@@ -1191,7 +1212,28 @@ class JaxprStackFrame:
     const_eqns = [eqn for eqn in self.eqns if set(eqn.invars) & constvars]
     return invar_positions, const_eqns
 
+def _prune_convert_element_types(jaxpr, constvals):
+  consts = dict(zip(jaxpr.constvars, constvals))
+  new_eqns = []
+  for eqn in jaxpr.eqns:
+    if eqn.primitive is core.convert_element_type_p:
+      c = consts.get(eqn.invars[0])
+      if type(c) in core.literalable_types and not np.shape(c):
+        # constant-fold dtype conversion of literals to be inlined
+        consts[eqn.outvars[0]] = np.array(c, eqn.params['new_dtype'])
+        continue
+      if c is not None and dtypes.dtype(c) == eqn.params['new_dtype']:
+        # don't stage out no-op convert_element_type calls as clutter
+        consts[eqn.outvars[0]] = c
+        continue
+    new_eqns.append(eqn)
+  new_constvars, new_constvals = unzip2(consts.items())
+  new_jaxpr = Jaxpr(new_constvars, jaxpr.invars, jaxpr.outvars, new_eqns)
+  return new_jaxpr, new_constvals
+
 def _inline_literals(jaxpr, constvals):
+  # This function also ensures variables are labeled in a canonical ordering,
+  # prunes unused constants, and inserts `dropvar` symbols.
   consts = dict(zip(jaxpr.constvars, constvals))
   newvar = core.gensym()
   newvars = {}
@@ -1205,20 +1247,17 @@ def _inline_literals(jaxpr, constvals):
       return None
 
   used = {v for eqn in jaxpr.eqns for v in eqn.invars} | set(jaxpr.outvars)
-  new_constvars = [var(v) for v in jaxpr.constvars if not lit(v)]
-  new_constvals = [c for v, c in zip(jaxpr.constvars, constvals) if not lit(v)]
+  new_constvars = [var(v) for v in jaxpr.constvars if v in used and not lit(v)]
+  new_constvals = [c for v, c in zip(jaxpr.constvars, constvals)
+                   if v in used and not lit(v)]
   new_invars = [var(v) for v in jaxpr.invars]
   new_eqns = []
   for eqn in jaxpr.eqns:
     invars = [lit(v) or var(v) for v in eqn.invars]
-    if (eqn.primitive is core.convert_element_type_p and type(invars[0]) is Literal):
-      # constant-fold dtype conversion of literals to be inlined
-      consts[eqn.outvars[0]] = np.array(invars[0].val, eqn.params['new_dtype'])
-    else:
-      # might do DCE here, but we won't until we're more careful about effects
-      outvars = [var(v) if v in used else dropvar for v in eqn.outvars]
-      new_eqns.append(new_jaxpr_eqn(invars, outvars, eqn.primitive, eqn.params,
-                                    eqn.source_info))
+    outvars = [var(v) if v in used else core.DropVar(v.aval)
+               for v in eqn.outvars]
+    new_eqns.append(new_jaxpr_eqn(invars, outvars, eqn.primitive, eqn.params,
+                                  eqn.source_info))
   new_outvars = [lit(v) or var(v) for v in jaxpr.outvars]
   new_jaxpr = Jaxpr(new_constvars, new_invars, new_outvars, new_eqns)
   return new_jaxpr, new_constvals
@@ -1319,7 +1358,7 @@ class DynamicJaxprTrace(core.Trace):
         jaxpr, reduced_out_avals, consts = trace_to_subjaxpr_dynamic(
             f, self.main, reduced_in_avals)
       out_axes = params['out_axes_thunk']()
-      out_avals = [core.unmapped_aval(params['axis_size'], out_axis, a)
+      out_avals = [core.unmapped_aval(axis_size, axis_name, out_axis, a)
                   if out_axis is not None else a
                   for a, out_axis in zip(reduced_out_avals, out_axes)]
       source_info = source_info_util.current()
