@@ -25,9 +25,9 @@ from . import partial_eval as pe
 from . import pxla
 from . import xla
 from .. import linear_util as lu
-from ..lib import xla_bridge as xb
-from ..lib import xla_client as xc
-from ..api_util import argnums_partial, flatten_axes, flatten_fun, _ensure_index_tuple
+from jax._src.lib import xla_bridge as xb
+from jax._src.lib import xla_client as xc
+from .._src.api_util import argnums_partial, flatten_axes, flatten_fun, _ensure_index_tuple
 from ..tree_util import tree_flatten, tree_unflatten
 from .._src.util import (extend_name_stack, wrap_name, wraps, safe_zip,
                          HashableFunction)
@@ -87,10 +87,10 @@ def _sharded_callable(
 
   jaxpr, global_out_avals, consts = pe.trace_to_jaxpr_final(fun, global_abstract_args)
 
-  if xb.get_backend().platform not in ["tpu", "gpu"]:
+  platform = xb.get_backend().platform
+  if platform  not in ["tpu", "gpu"]:
     # TODO(skye): fall back to regular jit?
-    raise ValueError("sharded_jit not supported for " +
-                     xb.get_backend().platform)
+    raise ValueError(f"sharded_jit not supported for {platform}")
 
   nparts = pxla.reconcile_num_partitions(jaxpr, nparts)
   assert nparts is not None
@@ -135,16 +135,16 @@ def _sharded_callable(
 
   log_priority = logging.WARNING if config.jax_log_compiles else logging.DEBUG
   logging.log(log_priority,
-              f"Compiling {fun.__name__} for {nparts} devices with "
-              f"args {global_abstract_args}.")
+              "Compiling %s for %d devices with args %s.",
+              fun.__name__, nparts, global_abstract_args)
 
-  c = xb.make_computation_builder("spjit_{}".format(fun.__name__))
+  c = xc.XlaBuilder("spjit_{}".format(fun.__name__))
   xla_consts = _map(partial(xb.constant, c), consts)
   xla_args = _xla_sharded_args(c, global_abstract_args, in_parts)
   axis_env = xla.AxisEnv(nrep, (), ())
-  out_nodes = xla.jaxpr_subcomp(
-      c, jaxpr, None, axis_env, xla_consts,
-      extend_name_stack(wrap_name(name, "sharded_jit")), *xla_args)
+  ctx = xla.TranslationContext(
+      c, platform, axis_env, extend_name_stack(wrap_name(name, "sharded_jit")))
+  out_nodes = xla.jaxpr_subcomp(ctx, jaxpr, xla_consts, *xla_args)
   out_tuple = xb.with_sharding(c, out_parts, xops.Tuple, c, out_nodes)
   built = c.Build(out_tuple)
 
@@ -194,9 +194,10 @@ def _sharded_jit_translation_rule(c, axis_env, in_nodes, name_stack,
     arg = xb.parameter(subc, i, c.GetShape(n))
     args.append(xb.set_sharding(subc, arg, sharding))
 
-  out_nodes = xla.jaxpr_subcomp(
-      subc, call_jaxpr, backend, axis_env, (),
-      extend_name_stack(name_stack, wrap_name(name, "sharded_jit")), *args)
+  ctx = xla.TranslationContext(
+      subc, backend, axis_env,
+      extend_name_stack(wrap_name(name, "sharded_jit")))
+  out_nodes = xla.jaxpr_subcomp(ctx, call_jaxpr, (), *args)
   out_parts = out_parts_thunk()
   assert len(out_parts) == len(out_nodes)
   out_nodes = [xb.set_sharding(subc, out, sharding)
