@@ -128,6 +128,34 @@ def make_op_metadata(primitive: core.Primitive,
 
 ### handlers
 
+_dtype_to_primitive_type: Dict[np.dtype, xc.PrimitiveType] = {
+  np.dtype('bool'): xc.PrimitiveType.PRED,
+  np.dtype('int8'): xc.PrimitiveType.S8,
+  np.dtype('int16'): xc.PrimitiveType.S16,
+  np.dtype('int32'): xc.PrimitiveType.S32,
+  np.dtype('int64'): xc.PrimitiveType.S64,
+  np.dtype('uint8'): xc.PrimitiveType.U8,
+  np.dtype('uint16'): xc.PrimitiveType.U16,
+  np.dtype('uint32'): xc.PrimitiveType.U32,
+  np.dtype('uint64'): xc.PrimitiveType.U64,
+  np.dtype(dtypes.bfloat16): xc.PrimitiveType.BF16,
+  np.dtype('float16'): xc.PrimitiveType.F16,
+  np.dtype('float32'): xc.PrimitiveType.F32,
+  np.dtype('float64'): xc.PrimitiveType.F64,
+  np.dtype('complex64'): xc.PrimitiveType.C64,
+  np.dtype('complex128'): xc.PrimitiveType.C128,
+}
+
+def dtype_to_primitive_type(dtype: np.dtype) -> xc.PrimitiveType:
+  """Converts a NumPy dtype into an XLA PrimitiveType."""
+  # Many things (e.g., strings, scalar types) can be compared with NumPy dtypes,
+  # but may not hash correctly. Make sure we have a true np.dtype.
+  assert isinstance(dtype, np.dtype), type(dtype)
+  try:
+    return _dtype_to_primitive_type[dtype]
+  except KeyError as err:
+    raise TypeError(f"No XLA lowering for NumPy dtype: {dtype}") from err
+
 xb.register_constant_handler(core.Unit, lambda c, *_: _make_unit_constant(c))
 
 def aval_to_xla_shapes(aval: core.AbstractValue) -> Sequence[XlaShape]:
