@@ -16,30 +16,30 @@
 
 # flake8: noqa: F401
 
-from jax.numpy import tanh
-from . import initializers
+from jax.numpy import tanh as tanh
+from . import initializers as initializers
 from jax._src.nn.functions import (
-  celu,
-  elu,
-  gelu,
-  glu,
-  hard_sigmoid,
-  hard_silu,
-  hard_swish,
-  hard_tanh,
-  leaky_relu,
-  log_sigmoid,
-  log_softmax,
-  logsumexp,
-  normalize,
-  one_hot,
-  relu,
-  relu6,
-  selu,
-  sigmoid,
-  soft_sign,
-  softmax,
-  softplus,
-  silu,
-  swish,
+  celu as celu,
+  elu as elu,
+  gelu as gelu,
+  glu as glu,
+  hard_sigmoid as hard_sigmoid,
+  hard_silu as hard_silu,
+  hard_swish as hard_swish,
+  hard_tanh as hard_tanh,
+  leaky_relu as leaky_relu,
+  log_sigmoid as log_sigmoid,
+  log_softmax as log_softmax,
+  logsumexp as logsumexp,
+  normalize as normalize,
+  one_hot as one_hot,
+  relu as relu,
+  relu6 as relu6,
+  selu as selu,
+  sigmoid as sigmoid,
+  soft_sign as soft_sign,
+  softmax as softmax,
+  softplus as softplus,
+  silu as silu,
+  swish as swish,
 )
