@@ -15,24 +15,24 @@
 # flake8: noqa: F401
 
 from jax._src.numpy.fft import (
-  ifft,
-  ifft2,
-  ifftn,
-  ifftshift,
-  ihfft,
-  irfft,
-  irfft2,
-  irfftn,
-  fft,
-  fft2,
-  fftfreq,
-  fftn,
-  fftshift,
-  hfft,
-  rfft,
-  rfft2,
-  rfftfreq,
-  rfftn,
+  ifft as ifft,
+  ifft2 as ifft2,
+  ifftn as ifftn,
+  ifftshift as ifftshift,
+  ihfft as ihfft,
+  irfft as irfft,
+  irfft2 as irfft2,
+  irfftn as irfftn,
+  fft as fft,
+  fft2 as fft2,
+  fftfreq as fftfreq,
+  fftn as fftn,
+  fftshift as fftshift,
+  hfft as hfft,
+  rfft as rfft,
+  rfft2 as rfft2,
+  rfftfreq as rfftfreq,
+  rfftn as rfftn,
 )
 
 # Module initialization is encapsulated in a function to avoid accidental
