@@ -15,6 +15,6 @@
 # flake8: noqa: F401
 
 from jax._src.scipy.stats.bernoulli import (
-  logpmf,
-  pmf,
-)
\ No newline at end of file
+  logpmf as logpmf,
+  pmf as pmf,
+)
